"use strict";

var _chalk = _interopRequireDefault(require("chalk"));

var _ejs = _interopRequireDefault(require("ejs"));

var _execa = _interopRequireDefault(require("execa"));

var _fsExtra = _interopRequireDefault(require("fs-extra"));

var _path = _interopRequireDefault(require("path"));

var _yargs = _interopRequireDefault(require("yargs"));

var _prompt = _interopRequireDefault(require("./helpers/prompt"));

var _summaryMessage = _interopRequireDefault(require("./helpers/summaryMessage"));

var _packages = _interopRequireDefault(require("./packages"));

var _types = require("./types");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const COMMON_FILES = _path.default.resolve(__dirname, '../templates/common');

const GRAPHQL_FILES = _path.default.resolve(__dirname, '../templates/graphql');

const REDUX_FILES = _path.default.resolve(__dirname, '../templates/redux'); // const FASTLANE_FILES = path.resolve(__dirname, '../templates/fastlane');


const args = {
  integrations: {
    description: 'Which integrations/features would you like to have in your app?',
    choices: [_types.Integrations.Fastlane, _types.Integrations.GraphQL, _types.Integrations.Redux, _types.Integrations.Unimodules]
  },
  modules: {
    description: 'Which packages would you like to have in your app?',
    choices: [_types.Modules.Config, _types.Modules.GestureHandler, _types.Modules.KeyboardManager, _types.Modules.Navigation, _types.Modules.Reanimated, _types.Modules.Screens, _types.Modules.Svg, _types.Modules.VectorIcons]
  }
};

async function create(argv) {
  const folder = _path.default.join(process.cwd(), argv.name);

  const basename = _path.default.basename(argv.name);

  const questions = {
    integrations: {
      type: 'multiselect',
      name: 'integrations',
      message: 'Which integrations/features would you like to have in your app?',
      choices: [{
        title: 'Redux',
        value: _types.Integrations.Redux
      }, {
        title: 'GraphQL',
        value: _types.Integrations.GraphQL
      }, {
        title: 'Unimodules',
        value: _types.Integrations.Unimodules
      } // { title: 'Fastlane', value: Integrations.Fastlane },
      // { title: 'Sentry', value: Integrations.Sentry },
      ]
    },
    modules: {
      type: 'multiselect',
      name: 'modules',
      message: 'Which packages would you like to add to your app?',
      choices: [{
        title: 'react-native-screens',
        value: _types.Modules.Screens
      }, {
        title: 'react-native-reanimated',
        value: _types.Modules.Reanimated
      }, {
        title: 'react-native-gesture-handler',
        value: _types.Modules.GestureHandler
      }, {
        title: 'react-native-svg',
        value: _types.Modules.Svg
      }, {
        title: 'react-native-vector-icons',
        value: _types.Modules.VectorIcons
      } // { title: 'react-navigation', value: Modules.Navigation },
      // { title: 'react-native-config', value: Modules.Config },
      // { title: 'react-native-keyboard-manager', value: Modules.KeyboardManager },
      ]
    }
  };
  const {
    integrations,
    modules
  } = { ...argv,
    ...(await (0, _prompt.default)(Object.entries(questions).filter(([k, v]) => !(argv[k] && v.validate ? v.validate(argv[k]) === true : Boolean(argv[k]))).map(([, v]) => v)))
  };

  if (!(await _fsExtra.default.pathExists(folder))) {
    console.log(_chalk.default.bgCyan.black(`Initialising react-native project ${basename}. It may take some time...`));

    _execa.default.sync('npx', ['react-native', 'init', basename, '--template', 'react-native-template-typescript']);

    console.log(_chalk.default.green('✅ Project initialised'));
  }
  /**
   * Creating options for ejs parser i.e:
   * { integrations: { redux: true }, modules: { screens: true } }
   * TODO: Maybe it's overcomplicated. Get back to it after some time :)
   */


  const options = Object.entries({
    integrations,
    modules
  }).reduce((options, [mainKey, values]) => {
    options[mainKey] = values.reduce((acc, key) => {
      acc[key] = true;
      return acc;
    }, {});
    return options;
  }, {
    project: {
      name: basename,
      package: basename.toLowerCase()
    }
  });

  const copyDir = async (source, dest) => {
    await _fsExtra.default.mkdirp(dest);
    const files = await _fsExtra.default.readdir(source);

    for (const f of files) {
      const target = _path.default.join(dest, _ejs.default.render(f.replace(/^\$/, '').replace('.ejs', ''), options, {
        openDelimiter: '{',
        closeDelimiter: '}'
      }));

      const file = _path.default.join(source, f);

      const stats = await _fsExtra.default.stat(file);

      if (stats.isDirectory()) {
        await copyDir(file, target);
      } else {
        const content = await _fsExtra.default.readFile(file, 'utf8');
        await _fsExtra.default.writeFile(target, _ejs.default.render(content, options));
      }
    }
  };

  const [packages, devPackages] = (0, _packages.default)(options);
  console.log(_chalk.default.blue('Coping common files'));
  await copyDir(COMMON_FILES, folder);
  console.log(_chalk.default.green('✅ Copied common files'));

  if (options.integrations.graphql) {
    console.log(_chalk.default.blue('Coping GraphQL files'));
    await copyDir(GRAPHQL_FILES, folder);
    console.log(_chalk.default.green('✅ Copied GraphQL files'));
  }

  if (options.integrations.redux) {
    console.log(_chalk.default.blue('Coping Redux files'));
    await copyDir(REDUX_FILES, folder);
    console.log(_chalk.default.green('✅ Copied Redux files'));
  } // if (options.integrations.fastlane) {
  //   console.log(chalk.blue('Coping Fastlane files'));
  //   await copyDir(FASTLANE_FILES, folder);
  //   console.log(chalk.green('✅ Copied Fastlane files'));
  // }

  /**
   * A bit hackish but for now I did not found anything better :shrug:
   */


  _execa.default.sync('chmod', ['+x', `${folder}/scripts/postinstall.sh`]);

  if (packages.length > 0) {
    console.log(_chalk.default.blue(`Adding dependencies ${packages.join(', ')}`));

    _execa.default.sync('yarn', ['--cwd', basename, 'add', '--ignore-scripts'].concat(packages));

    console.log(_chalk.default.green('✅ Dependencies added'));
  }

  if (devPackages.length > 0) {
    console.log(_chalk.default.blue(`Adding dev dependencies ${devPackages.join(', ')}`));

    _execa.default.sync('yarn', ['--cwd', basename, 'add', '--dev', '--ignore-scripts'].concat(devPackages));

    console.log(_chalk.default.green('✅ Dev dependencies added'));
  }

  console.log(_chalk.default.blue(`Installing dependencies & Pods...`));

  _execa.default.sync('yarn', ['--cwd', basename, 'install']);

  console.log(_chalk.default.green('✅ Dependencies & Pods installed'));
  console.log(_chalk.default.blue(`Linting...`));

  _execa.default.sync('yarn', ['--cwd', basename, 'lint', '--fix']);

  console.log(_chalk.default.green('✅ Lint successful'));
  console.log((0, _summaryMessage.default)(basename));
}

_yargs.default.command('$0 <name>', 'create a react native library', args, create).demandCommand().recommendCommands().strict().argv;
//# sourceMappingURL=index.js.map